// query et traitement des contenus

import { defineStore } from 'pinia';
import REST from '../api/rest-axios';

export const useContentStore = defineStore('content', {
    state: () => ({
        contentType: '',
        pageTitle: '',
        content: {
            contentTitle: '',
            coordinates: {},
            adresse: {},
            etape_number: '',
            couleur: '',
            dates: {},
            previous : {},
            next: {},
            vignette: {},
            parties: [],
        },
        loading: false,
        error: null,
    }),
    actions: {
        async fetchContentData(path) {
            this.resetStore(false);
            const contentTypes = [ 'etape', 'static' ];
            try {
                let rawContent, 
                    contentType,
                    response;

                contentTypesLoop:
                for (let type of contentTypes) {
                    response = await REST.get(`/jsonapi/node/${type}/`);
                    for (let content of response.data.data) {
                        for (let tag of content.attributes.metatag) {
                            if (tag.tag === "link" && tag.attributes.href === path) {
                                this.contentType = type;
                                rawContent = content;
                                contentType = type;
                                break contentTypesLoop;
                            }
                        }
                    }
                }
                // pageTitle
                for (let tag of rawContent.attributes.metatag) {
                    if (tag.tag === "meta") {
                        this.pageTitle = tag.attributes.content;
                        break;
                    }
                }
                // contentTitle
                this.content.contentTitle = rawContent.attributes.title;

                // vignette
                const vignetteFetch = await this.fetchDeeperContent('field_vignette', rawContent.relationships);
                if (vignetteFetch) {
                    this.content.vignette = { 
                        url: vignetteFetch.attributes.uri.url,
                        alt: rawContent.relationships.field_vignette.data.meta.alt
                    };
                }                

                if (contentType === 'etape') {
                    // coordinates
                    this.content.coordinates = {
                        lat: rawContent.attributes.field_geofield.lat,
                        lon: rawContent.attributes.field_geofield.lon,
                    };
                    // adresse
                    this.content.adresse = rawContent.attributes.field_adresse;
                    // étape number
                    this.content.etape_number = rawContent.attributes.field_arret_numero;
                    // couleur
                    this.content.couleur = rawContent.attributes.field_couleur;
                    // dates
                    this.content.dates = {
                        start: this.getCleanDate(rawContent.attributes.field_dates.value),
                        end: this.getCleanDate(rawContent.attributes.field_dates.end_value),
                    }
                    // previous / next
                    await this.getRelatedEtape('previous', response.data.data, path);
                    await this.getRelatedEtape('next', response.data.data, path);
                }                

                // parties
                const fieldParties = contentType === 'etape' ? 'field_parties' : 'field_parties_static';
                const partiesFetch = await this.fetchDeeperContent(fieldParties, rawContent.relationships);
                
                if (partiesFetch) {
                    this.content.parties = [];
                    for (let partie of partiesFetch) {
                        const partieType = partie.type.replace(/^paragraph--/, "");
                        let partieContent = {
                            type: partieType,
                        };

                        switch (partieType) {
                            case 'carte_sensible':
                                const carteSensibleFetch = await this.fetchDeeperContent('field_image_carte', partie.relationships);                                
                                if (carteSensibleFetch) {
                                    partieContent.carteSensible = {
                                        url: carteSensibleFetch.attributes.uri.url,
                                        alt: partie.relationships.field_image_carte.data.meta.alt,
                                    };
                                }
                                break;
                            case 'titre_texte':
                                partieContent.titre = partie.attributes.field_titre;
                                partieContent.texte = partie.attributes.field_texte.value;                                        
                                break;
                            case 'chiffres_cles':
                                const chiffresClesFetch = await this.fetchDeeperContent('field_chiffres_clefs', partie.relationships);
                                if (chiffresClesFetch) {
                                    partieContent.chiffresCles = [];
                                    for (let chiffre of chiffresClesFetch) {
                                        partieContent.chiffresCles.push({
                                            chiffre: chiffre.attributes.field_chiffre,
                                            description: chiffre.attributes.field_description,
                                        });
                                    }
                                }
                                break;
                            case 'diaporama':
                                const diaporamaFetch = await this.fetchDeeperContent('field_diaporama', partie.relationships);
                                if (diaporamaFetch) {
                                    partieContent.diaporama = [];
                                    for (let [index, image] of diaporamaFetch.entries()) {
                                        partieContent.diaporama.push({
                                            url: image.attributes.uri.url,
                                            alt: partie.relationships.field_diaporama.data[index].meta.alt,
                                        });
                                    }
                                }
                                break;
                            case 'entretien':
                                partieContent.entretien = {};
                                const personnesFetch = await this.fetchDeeperContent('field_personne_s', partie.relationships);
                                const questionsReponsesFetch = await this.fetchDeeperContent('field_questions_reponses', partie.relationships);
                                if (personnesFetch && questionsReponsesFetch) {
                                    partieContent.entretien.personnes = [];
                                    for (let personne of personnesFetch) {
                                        const portraitFetch = await this.fetchDeeperContent('field_portrait', personne.relationships);
                                        if (portraitFetch) {                                                    
                                            partieContent.entretien.personnes.push({
                                                portrait: portraitFetch.attributes.uri.url,
                                                alt: personne.relationships.field_portrait.data.meta.alt,
                                                description: personne.attributes.field_description,
                                            });
                                        }
                                    }
                                    partieContent.entretien.questionsReponses = [];
                                    for (let qr of questionsReponsesFetch) {
                                        partieContent.entretien.questionsReponses.push({
                                            question: qr.attributes.field_question,
                                            reponse: qr.attributes.field_reponse.value,
                                        });
                                    }                                           
                                }
                                break;
                            case 'exergue':
                                partieContent.exergue = partie.attributes.field_texte_exergue.value;
                                break;
                            case 'video':
                                partieContent.videos = [];
                                for (let video of partie.attributes.field_videos) {
                                    const videoId = video.split('?v=')[1];
                                    const videoUrl = `https://www.youtube.com/embed/${videoId}`;
                                    partieContent.videos.push(videoUrl);                      
                                }
                                break;
                        }
                        this.content.parties.push(partieContent);
                    }
                }
            } catch (error) {
                this.error = 'Failed to fetch data';
                console.error('Issue with getNodeData', error);
            } finally {               
                this.loading = false;                
            }
        },
        getCleanDate(date) {
            return {
              d: date.split('-')[2],
              m: new Intl.DateTimeFormat('fr-FR', { month: 'long' }).format(new Date(date)),
              y: date.split('-')[0],
            }
        },
        async getRelatedEtape(direction, allEtapesData, path) {
            const getRelatedEtapeContent = async (relatedPath, allEtapesData) => {
                const baseUrl = window.location.protocol + "//" + window.location.host;
                for (let etape of allEtapesData) {
                    for (let tag of etape.attributes.metatag) {
                        if (tag.tag === "link" && tag.attributes.href === baseUrl + relatedPath) {                            
                            const vignetteFetch = await REST.get(etape.relationships.field_vignette.links.related.href);
                            this.content[direction] = {
                                url: tag.attributes.href,
                                couleur: etape.attributes.field_couleur,
                                title: etape.attributes.title,
                                postalCode: etape.attributes.field_adresse.postal_code,
                                dates: {
                                    start: this.getCleanDate(etape.attributes.field_dates.value),
                                    end: this.getCleanDate(etape.attributes.field_dates.end_value),
                                },
                                vignette: {
                                    url: vignetteFetch.data.data.attributes.uri.url,
                                    alt: etape.relationships.field_vignette.data.meta.alt,
                                },
                            }
                        }
                    }
                }
            }

            const orderedEtapesList = document.querySelectorAll('#etapes-liste li');
            if (orderedEtapesList) {
                for (let li of orderedEtapesList) {
                    const liHref = li.querySelector('a').getAttribute('href');
                    if (path.endsWith(liHref)) {
                        const previousEtapeItemPath = li.previousElementSibling?.querySelector('a').getAttribute('href');
                        const nextEtapeItemPath = li.nextElementSibling?.querySelector('a').getAttribute('href');
                        
                        if (previousEtapeItemPath && direction === 'previous') {
                            let prevContent = await getRelatedEtapeContent(previousEtapeItemPath, allEtapesData);
                            return prevContent;
                        }
                        if (nextEtapeItemPath && direction === 'next') {
                            let nextContent = await getRelatedEtapeContent(nextEtapeItemPath, allEtapesData);
                            return nextContent;
                        }
                    }
                }
            }
        },
        async fetchDeeperContent(field, relationships) {
            if (relationships[field].data) {
              try {
                const contentLink = relationships[field].links.related.href;
                const contentFetch = await REST.get(contentLink);
                return contentFetch.data.data;
              } catch (error) {
                this.error = 'Failed to fetch data';
                console.error('Issue with getNodeData', error);
              }
            }
        },
        resetStore(forFrontDisplay) {
            this.contentType = '';
            this.pageTitle = '';
            this.content = {};
            this.loading = !forFrontDisplay;
            this.error = null;
        }
    },
});